/************************************************************************
 *   IRC - Internet Relay Chat, src/m_stopic.c
 *   Copyright (C) 1990 Jarkko Oikarinen and
 *                      University of Oulu, Computing Center
 *
 *   Written by Johnie Ingram (netgod) for Open Projects.
 *
 *   See file AUTHORS in IRC package for additional names of
 *   the programmers. 
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 1, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *   
 */

#include "client.h"
#include "channel.h"
#include "hash.h"
#include "struct.h"
#include "ircd.h"
#include "numeric.h"
#include "s_serv.h"
#include "send.h"
#include "list.h"
#include "irc_string.h"
#include "msg.h"
#include "handlers.h"
#include <stdlib.h>

extern int m_stopic(struct Client *, struct Client *, int, char **);

struct Message stopic_msgtab = {
  "STOPIC", 0, 4, 0, MFLG_SLOW, 0,
  {m_unregistered, m_ignore, m_stopic},
  NULL, NULL
};

/*
 * m_functions execute protocol messages on this server:
 *
 *      cptr    is always NON-NULL, pointing to a *LOCAL* client
 *              structure (with an open socket connected!). This
 *              identifies the physical socket where the message
 *              originated (or which caused the m_function to be
 *              executed--some m_functions may call others...).
 *
 *      sptr    is the source of the message, defined by the
 *              prefix part of the message if present. If not
 *              or prefix not found, then sptr==cptr.
 *
 *              (!IsServer(cptr)) => (cptr == sptr), because
 *              prefixes are taken *only* from servers...
 *
 *              (IsServer(cptr))
 *                      (sptr == cptr) => the message didn't
 *                      have the prefix.
 *
 *                      (sptr != cptr && IsServer(sptr) means
 *                      the prefix specified servername. (?)
 *
 *                      (sptr != cptr && !IsServer(sptr) means
 *                      that message originated from a remote
 *                      user (not local).
 *
 *              combining
 *
 *              (!IsServer(sptr)) means that, sptr can safely
 *              taken as defining the target structure of the
 *              message in this server.
 *
 *      *Always* true (if 'parse' and others are working correct):
 *
 *      1)      sptr->from == cptr  (note: cptr->from == cptr)
 *
 *      2)      MyConnect(sptr) <=> sptr == cptr (e.g. sptr
 *              *cannot* be a local connection, unless it's
 *              actually cptr!). [MyConnect(x) should probably
 *              be defined as (x == x->from) --msa ]
 *
 *      parc    number of variable parameter strings (if zero,
 *              parv is allowed to be NULL)
 *
 *      parv    a NULL terminated list of parameter pointers,
 *
 *                      parv[0], sender (prefix string), if not present
 *                              this points to an empty string.
 *                      parv[1]...parv[parc-1]
 *                              pointers to additional parameters
 *                      parv[parc] == NULL, *always*
 *
 *              note:   it is guaranteed that parv[0]..parv[parc-1] are all
 *                      non-NULL pointers.
 */



/*
** m_stopic
**      parv[0] = sender prefix
**      parv[1] = channel name
**      parv[2] = topic nick
**      parv[3] = topic time
**      parv[4] = topic text
*/

int
m_stopic(struct Client *cptr, struct Client *sptr, int parc, char *parv[])
{
  struct Channel *chptr = NullChn;

  if (parc < 5)
    {
      sendto_one(sptr, form_str(ERR_NEEDMOREPARAMS),
		 me.name, parv[0], "STOPIC");
      return 0;
    }
  if (!IsServer(sptr))
    {
      sendto_one(sptr, form_str(ERR_NOPRIVILEGES), me.name, parv[0]);
      return (0);
    }
#ifdef TOPIC_INFO
  if ((chptr = hash_find_channel(parv[1], NullChn)))
    {
      time_t topic_time = atoi(parv[3]);
      if (!topic_time || topic_time < chptr->topic_time)
	{
	  chptr->topic_time = (topic_time) ? topic_time : CurrentTime;
	  strncpy_irc(chptr->topic_nick, parv[2], NICKLEN);
	  strncpy_irc(chptr->topic, parv[4], TOPICLEN);
	  sendto_channel_butserv(chptr, sptr, ":%s TOPIC %s :%s",
				 parv[0], chptr->chname, chptr->topic);
	}
    }
#endif
  sendto_serv_butone(cptr, ":%s STOPIC %s %s %s :%s",
		     parv[0], parv[1], parv[2], parv[3], parv[4]);
  return 0;
}
